@@ -1,8 +1,13 @@
 package M9;
 use Object::Simple;
 
+sub new {
+    bless { m1 => 10 }, 'M9';
+}
+
 sub m1 : Attr { default => 3 }
 
+sub m3 : Attr { default => 2 }
 sub m4 {
     return 2;
 }
